'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint global-require: "off" */
var _require = require('funsee-logger'),
    FunSeeLogger = _require.FunSeeLogger;

var _require2 = require('./tool/initial'),
    validateFunSeeConfig = _require2.validateFunSeeConfig;

var combineServerRoute = require('./tool/combineServerRoute');

var createApp = require('./tool/appCreator');
var routerCreator = require('./tool/routerCreator');
var middlewareAppender = require('./tool/middlewareAppender');
var configLoader = require('./tool/configLoader');
var attachService = require('./tool/attachService');

var _require3 = require('../package.json'),
    version = _require3.version;

var _config = Symbol('config');
var _app = Symbol('app');
var _serverRouters = Symbol('serverRouters');
var _logger = Symbol('logger');

var FunSee = function () {
  /**
   * FunSee构造函数
   *
   * FunSee采用单例模式，可接受直接传入funsee配置， 传入的配置将会merge到defaultConfig里面
   *
   * @param    {object}  config     FunSee的config， 详见defaultFunSeeConfig.js
   * @returns  FunSee               返回FunSee的实例（单例)
   *
   * @date     2018-03-08
   * @author   yanzhenggang<robinyzg@hotmail.com>
   */
  function FunSee(config) {
    _classCallCheck(this, FunSee);

    if (FunSee.prototype.__instance === undefined) {
      this.initializeFunSee(config);
      FunSee.prototype.__instance = this;
    }

    return FunSee.prototype.__instance;
  }

  _createClass(FunSee, [{
    key: 'initializeFunSee',
    value: function initializeFunSee(config) {
      var _this = this;

      // 根据环境加载不同的config
      this[_config] = configLoader(config);

      // valid the config file for FunSee
      validateFunSeeConfig(this.config);

      // add log for Funsee instance
      this[_logger] = new FunSeeLogger(this.config.log || {});
      // attach logger into Koa
      this.config.middleware.forEach(function (element) {
        if (element.desc === 'FunSeeLogger') {
          element.param = _this.config.log;
        }
      });

      this.logger.silly('FunSee: ' + version + ': Init');

      // set Koa App
      this[_app] = createApp();

      this[_app].keys = ['Funsee is a really good project'];
      this[_app].context.fsConfig = this[_app].context.fsConfig || this[_config].customParam || {};

      // load important file for Funsee
      this.importantFileLoader();

      // add service into Funsee
      attachService(this.config, this.app);

      // load midderware for KOA app
      middlewareAppender(this);

      // add router
      routerCreator(this.app, this.serverRouters);
    }
  }, {
    key: 'importantFileLoader',
    value: function importantFileLoader() {
      // collect server router
      var routeConfig = combineServerRoute();
      this[_serverRouters] = routeConfig;
    }
  }, {
    key: 'startup',
    value: function startup() {
      this.app.listen(this.config.port);
      this.logger.info('listening at port: ', this.config.port);
    }
  }, {
    key: 'use',
    value: function use(mid) {
      this.app.use(mid);
    }
  }, {
    key: 'logger',
    get: function get() {
      return this[_logger];
    }
  }, {
    key: 'config',
    get: function get() {
      return this[_config];
    }
  }, {
    key: 'app',
    get: function get() {
      return this[_app];
    }
  }, {
    key: 'serverRouters',
    get: function get() {
      return this[_serverRouters];
    }
  }]);

  return FunSee;
}();

FunSee.connect = require('./tool/connector');

module.exports = FunSee;