'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var Router = require('koa-router');

var router = new Router();

router.get('/health', function (ctx) {
  ctx.status = 200;
});

/**
 * 函数功能简述
 *
 * 处理收集到的路由配置
 *
 * @param    {Objedt}  controller     路由配置
 * @param    {String}  prefix     以路由文件文件名命名的前置路由
 * @returns  void
 *
 * @date     2018-3-1
 * @author   zhaoxing<jiyiwohanxing@gmail.com>
 */
var getController = function getController(controller, prefix) {
  if (_typeof(controller[controller.length - 1]) === 'object') controller.pop();

  return controller.map(function (item, idx) {
    if (idx === 0) return prefix + item;
    var file = require(global.__ROOT_PATH__ + '/server/controller/' + item.split('.').shift());
    return file[item.split('.').pop()];
  });
};

/**
 * 函数功能简述
 *
 * 挂载总路由
 *
 * @param    {Objedt}  app     Koa App
 * @param    {Array}  routers     合并处理后的服务器端路由配置
 * @returns  void
 *
 * @date     2018-2-28
 * @author   zhaoxing<jiyiwohanxing@gmail.com>
 */
module.exports = function (app, routers) {
  var prefixRouter = new Router({ prefix: '/api' });

  routers.forEach(function (element) {
    var RouterGenerator = {

      post: function post() {
        for (var _len = arguments.length, arg = Array(_len), _key = 0; _key < _len; _key++) {
          arg[_key] = arguments[_key];
        }

        var routerController = getController(arg, element.name);
        prefixRouter.post.apply(prefixRouter, _toConsumableArray(routerController));
      },

      get: function get() {
        for (var _len2 = arguments.length, arg = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          arg[_key2] = arguments[_key2];
        }

        var routerController = getController(arg, element.name);
        prefixRouter.get.apply(prefixRouter, _toConsumableArray(routerController));
      }
    };

    element.router(RouterGenerator);
  });

  router.use(prefixRouter.routes(), prefixRouter.allowedMethods());

  app.use(router.routes(), router.allowedMethods());
};